@@ -18,9 +18,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
  */
 
 /**
@@ -355,10 +353,7 @@ clutter_behaviour_class_init (ClutterBehaviourClass *klass)
 static void
 clutter_behaviour_init (ClutterBehaviour *self)
 {
-  ClutterBehaviourPrivate *priv;
-
-  self->priv = priv = CLUTTER_BEHAVIOUR_GET_PRIVATE (self);
-
+  self->priv = CLUTTER_BEHAVIOUR_GET_PRIVATE (self);
 }
 
 static void
