@@ -16,9 +16,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ *
+ *
  */
 
 #if !defined(__COGL_H_INSIDE__) && !defined(CLUTTER_COMPILATION)
@@ -86,12 +86,12 @@ typedef enum {
  *
  * Return value: a handle to the new material
  */
-CoglHandle cogl_material_new (void);
-
-#ifndef COGL_DISABLE_DEPRECATED
+CoglHandle
+cogl_material_new (void);
 
 /**
  * cogl_material_copy:
+ * @source: the handle for the material to copy
  *
  * Creates a new material handle with the configuration copied from the
  * source material.
@@ -106,7 +106,10 @@ CoglHandle cogl_material_new (void);
  *
  * Since: 1.2
  */
-CoglHandle cogl_material_copy (CoglHandle source);
+CoglHandle
+cogl_material_copy (CoglHandle source);
+
+#ifndef COGL_DISABLE_DEPRECATED
 
 /**
  * cogl_material_ref:
@@ -120,7 +123,8 @@ CoglHandle cogl_material_copy (CoglHandle source);
  *
  * Deprecated: 1.2: Use cogl_handle_ref() instead
  */
-CoglHandle cogl_material_ref (CoglHandle handle);
+CoglHandle
+cogl_material_ref (CoglHandle handle) G_GNUC_DEPRECATED;
 
 /**
  * cogl_material_unref:
@@ -132,7 +136,8 @@ CoglHandle cogl_material_ref (CoglHandle handle);
  *
  * Deprecated: 1.2: Use cogl_handle_unref() instead
  */
-void cogl_material_unref (CoglHandle handle);
+void
+cogl_material_unref (CoglHandle handle) G_GNUC_DEPRECATED;
 
 #endif /* COGL_DISABLE_DEPRECATED */
 
@@ -145,7 +150,8 @@ void cogl_material_unref (CoglHandle handle);
  * Return value: %TRUE if the handle references a #CoglMaterial,
  *   %FALSE otherwise
  */
-gboolean cogl_is_material (CoglHandle handle);
+gboolean
+cogl_is_material (CoglHandle handle);
 
 /**
  * cogl_material_set_color:
@@ -163,8 +169,9 @@ gboolean cogl_is_material (CoglHandle handle);
  *
  * Since: 1.0
  */
-void cogl_material_set_color (CoglHandle       material,
-                              const CoglColor *color);
+void
+cogl_material_set_color (CoglHandle       material,
+                         const CoglColor *color);
 
 /**
  * cogl_material_set_color4ub:
@@ -180,11 +187,12 @@ void cogl_material_set_color (CoglHandle       material,
  *
  * Since: 1.0
  */
-void cogl_material_set_color4ub (CoglHandle material,
-			         guint8     red,
-                                 guint8     green,
-                                 guint8     blue,
-                                 guint8     alpha);
+void
+cogl_material_set_color4ub (CoglHandle material,
+			    guint8     red,
+                            guint8     green,
+                            guint8     blue,
+                            guint8     alpha);
 
 /**
  * cogl_material_set_color4f:
@@ -200,11 +208,12 @@ void cogl_material_set_color4ub (CoglHandle material,
  *
  * Since: 1.0
  */
-void cogl_material_set_color4f (CoglHandle material,
-                                float      red,
-                                float      green,
-                                float      blue,
-                                float      alpha);
+void
+cogl_material_set_color4f (CoglHandle material,
+                           float      red,
+                           float      green,
+                           float      blue,
+                           float      alpha);
 
 /**
  * cogl_material_get_color:
@@ -215,8 +224,9 @@ void cogl_material_set_color4f (CoglHandle material,
  *
  * Since: 1.0
  */
-void cogl_material_get_color (CoglHandle  material,
-                              CoglColor  *color);
+void
+cogl_material_get_color (CoglHandle  material,
+                         CoglColor  *color);
 
 /**
  * cogl_material_set_ambient:
@@ -234,8 +244,9 @@ void cogl_material_get_color (CoglHandle  material,
  *
  * Since: 1.0
  */
-void cogl_material_set_ambient (CoglHandle       material,
-				const CoglColor *ambient);
+void
+cogl_material_set_ambient (CoglHandle       material,
+			   const CoglColor *ambient);
 
 /**
  * cogl_material_get_ambient:
@@ -246,8 +257,9 @@ void cogl_material_set_ambient (CoglHandle       material,
  *
  * Since: 1.0
  */
-void cogl_material_get_ambient (CoglHandle  material,
-                                CoglColor  *ambient);
+void
+cogl_material_get_ambient (CoglHandle  material,
+                           CoglColor  *ambient);
 
 /**
  * cogl_material_set_diffuse:
@@ -262,8 +274,9 @@ void cogl_material_get_ambient (CoglHandle  material,
  *
  * Since: 1.0
  */
-void cogl_material_set_diffuse (CoglHandle       material,
-				const CoglColor *diffuse);
+void
+cogl_material_set_diffuse (CoglHandle       material,
+			   const CoglColor *diffuse);
 
 /**
  * cogl_material_get_diffuse:
@@ -274,8 +287,9 @@ void cogl_material_set_diffuse (CoglHandle       material,
  *
  * Since: 1.0
  */
-void cogl_material_get_diffuse (CoglHandle  material,
-                                CoglColor  *diffuse);
+void
+cogl_material_get_diffuse (CoglHandle  material,
+                           CoglColor  *diffuse);
 
 /**
  * cogl_material_set_ambient_and_diffuse:
@@ -291,8 +305,9 @@ void cogl_material_get_diffuse (CoglHandle  material,
  *
  * Since: 1.0
  */
-void cogl_material_set_ambient_and_diffuse (CoglHandle       material,
-					    const CoglColor *color);
+void
+cogl_material_set_ambient_and_diffuse (CoglHandle       material,
+				       const CoglColor *color);
 
 /**
  * cogl_material_set_specular:
@@ -307,8 +322,9 @@ void cogl_material_set_ambient_and_diffuse (CoglHandle       material,
  *
  * Since: 1.0
  */
-void cogl_material_set_specular (CoglHandle       material,
-				 const CoglColor *specular);
+void
+cogl_material_set_specular (CoglHandle       material,
+			    const CoglColor *specular);
 
 /**
  * cogl_material_get_specular:
@@ -319,24 +335,28 @@ void cogl_material_set_specular (CoglHandle       material,
  *
  * Since: 1.0
  */
-void cogl_material_get_specular (CoglHandle  material,
-                                 CoglColor  *specular);
+void
+cogl_material_get_specular (CoglHandle  material,
+                            CoglColor  *specular);
 
 /**
  * cogl_material_set_shininess:
  * @material: A CoglMaterial object
- * @shininess: The desired shininess; range: [0.0, 1.0]
+ * @shininess: The desired shininess; must be >= 0.0
  *
- * Sets the materials shininess, in the standard OpenGL lighting model,
- * which determines how specular highlights are calculated. A higher
- * @shininess will produce smaller brigher highlights.
+ * Sets the shininess of the material, in the standard OpenGL lighting
+ * model, which determines the size of the specular highlights. A
+ * higher @shininess will produce smaller highlights which makes the
+ * object appear more shiny.
  *
  * The default value is 0.0
  *
  * Since: 1.0
  */
-void cogl_material_set_shininess (CoglHandle material,
-				  float      shininess);
+void
+cogl_material_set_shininess (CoglHandle material,
+			     float      shininess);
+
 /**
  * cogl_material_get_shininess:
  * @material: A CoglMaterial object
@@ -347,7 +367,8 @@ void cogl_material_set_shininess (CoglHandle material,
  *
  * Since: 1.0
  */
-float cogl_material_get_shininess (CoglHandle material);
+float
+cogl_material_get_shininess (CoglHandle material);
 
 /**
  * cogl_material_set_emission:
@@ -362,8 +383,9 @@ float cogl_material_get_shininess (CoglHandle material);
  *
  * Since: 1.0
  */
-void cogl_material_set_emission (CoglHandle       material,
-				 const CoglColor *emission);
+void
+cogl_material_set_emission (CoglHandle       material,
+			    const CoglColor *emission);
 
 /**
  * cogl_material_get_emission:
@@ -374,8 +396,9 @@ void cogl_material_set_emission (CoglHandle       material,
  *
  * Since: 1.0
  */
-void cogl_material_get_emission (CoglHandle material,
-                                 CoglColor *emission);
+void
+cogl_material_get_emission (CoglHandle material,
+                            CoglColor *emission);
 
 /**
  * CoglMaterialAlphaFunc:
@@ -427,9 +450,10 @@ typedef enum {
  *
  * Since: 1.0
  */
-void cogl_material_set_alpha_test_function (CoglHandle            material,
-					    CoglMaterialAlphaFunc alpha_func,
-					    float                 alpha_reference);
+void
+cogl_material_set_alpha_test_function (CoglHandle            material,
+				       CoglMaterialAlphaFunc alpha_func,
+				       float                 alpha_reference);
 
 /**
  * cogl_material_set_blend:
@@ -515,9 +539,10 @@ void cogl_material_set_alpha_test_function (CoglHandle            material,
  *
  * Since: 1.0
  */
-gboolean cogl_material_set_blend (CoglHandle  material,
-                                  const char *blend_string,
-                                  GError    **error);
+gboolean
+cogl_material_set_blend (CoglHandle  material,
+                         const char *blend_string,
+                         GError    **error);
 
 /**
  * cogl_material_set_blend_constant:
@@ -529,8 +554,9 @@ gboolean cogl_material_set_blend (CoglHandle  material,
  *
  * Since: 1.0
  */
-void cogl_material_set_blend_constant (CoglHandle material,
-                                       CoglColor *constant_color);
+void
+cogl_material_set_blend_constant (CoglHandle material,
+                                  CoglColor *constant_color);
 
 /**
  * cogl_material_set_layer:
@@ -551,9 +577,10 @@ void cogl_material_set_blend_constant (CoglHandle material,
  *
  * Since: 1.0
  */
-void cogl_material_set_layer (CoglHandle material,
-			      int        layer_index,
-			      CoglHandle texture);
+void
+cogl_material_set_layer (CoglHandle material,
+			 int        layer_index,
+			 CoglHandle texture);
 
 /**
  * cogl_material_remove_layer:
@@ -562,8 +589,9 @@ void cogl_material_set_layer (CoglHandle material,
  *
  * This function removes a layer from your material
  */
-void cogl_material_remove_layer (CoglHandle material,
-				 gint       layer_index);
+void
+cogl_material_remove_layer (CoglHandle material,
+			    int        layer_index);
 
 
 /**
@@ -660,7 +688,7 @@ void cogl_material_remove_layer (CoglHandle material,
  */
 gboolean
 cogl_material_set_layer_combine (CoglHandle   material,
-				 gint         layer_index,
+				 int          layer_index,
 				 const char  *blend_string,
                                  GError     **error);
 
@@ -676,9 +704,10 @@ cogl_material_set_layer_combine (CoglHandle   material,
  *
  * Since: 1.0
  */
-void cogl_material_set_layer_combine_constant (CoglHandle  material,
-                                               int         layer_index,
-                                               CoglColor  *constant);
+void
+cogl_material_set_layer_combine_constant (CoglHandle  material,
+                                          int         layer_index,
+                                          CoglColor  *constant);
 
 /**
  * cogl_material_set_layer_matrix:
@@ -689,9 +718,10 @@ void cogl_material_set_layer_combine_constant (CoglHandle  material,
  * This function lets you set a matrix that can be used to e.g. translate
  * and rotate a single layer of a material used to fill your geometry.
  */
-void cogl_material_set_layer_matrix (CoglHandle  material,
-				     int         layer_index,
-				     CoglMatrix *matrix);
+void
+cogl_material_set_layer_matrix (CoglHandle  material,
+				int         layer_index,
+				CoglMatrix *matrix);
 
 /**
  * cogl_material_get_layers:
@@ -705,7 +735,8 @@ void cogl_material_set_layer_matrix (CoglHandle  material,
  *   functions. The list is owned by COGL and it  should not be modified or
  *   freed
  */
-G_CONST_RETURN GList *cogl_material_get_layers (CoglHandle material);
+G_CONST_RETURN GList *
+cogl_material_get_layers (CoglHandle material);
 
 /**
  * cogl_material_get_n_layers:
@@ -717,7 +748,8 @@ G_CONST_RETURN GList *cogl_material_get_layers (CoglHandle material);
  *
  * Since: 1.0
  */
-int cogl_material_get_n_layers (CoglHandle material);
+int
+cogl_material_get_n_layers (CoglHandle material);
 
 /**
  * CoglMaterialLayerType:
@@ -747,7 +779,8 @@ typedef enum {
  *
  * Return value: the type of the layer
  */
-CoglMaterialLayerType cogl_material_layer_get_type (CoglHandle layer);
+CoglMaterialLayerType
+cogl_material_layer_get_type (CoglHandle layer);
 
 /**
  * cogl_material_layer_get_texture:
@@ -763,7 +796,8 @@ CoglMaterialLayerType cogl_material_layer_get_type (CoglHandle layer);
  *
  * Return value: a #CoglHandle for the texture inside the layer
  */
-CoglHandle cogl_material_layer_get_texture (CoglHandle layer);
+CoglHandle
+cogl_material_layer_get_texture (CoglHandle layer);
 
 /**
  * cogl_material_layer_get_min_filter:
@@ -773,7 +807,8 @@ CoglHandle cogl_material_layer_get_texture (CoglHandle layer);
  *
  * Return value: the current downscaling filter
  */
-CoglMaterialFilter cogl_material_layer_get_min_filter (CoglHandle layer);
+CoglMaterialFilter
+cogl_material_layer_get_min_filter (CoglHandle layer);
 
 /**
  * cogl_material_layer_get_mag_filter:
@@ -783,7 +818,8 @@ CoglMaterialFilter cogl_material_layer_get_min_filter (CoglHandle layer);
  *
  * Return value: the current downscaling filter
  */
-CoglMaterialFilter cogl_material_layer_get_mag_filter (CoglHandle layer);
+CoglMaterialFilter
+cogl_material_layer_get_mag_filter (CoglHandle layer);
 
 /**
  * cogl_material_set_layer_filters:
@@ -795,10 +831,11 @@ CoglMaterialFilter cogl_material_layer_get_mag_filter (CoglHandle layer);
  * Changes the decimation and interpolation filters used when a texture is
  * drawn at other scales than 100%.
  */
-void cogl_material_set_layer_filters (CoglHandle         material,
-                                      gint               layer_index,
-                                      CoglMaterialFilter min_filter,
-                                      CoglMaterialFilter mag_filter);
+void
+cogl_material_set_layer_filters (CoglHandle         material,
+                                 int                layer_index,
+                                 CoglMaterialFilter min_filter,
+                                 CoglMaterialFilter mag_filter);
 
 G_END_DECLS
 
