@@ -15,9 +15,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ *
+ *
  *
  * Authored by:
  *      Matthew Allum <mallum@openedhand.com>
@@ -419,9 +419,9 @@ event_translate (ClutterBackend *backend,
   ClutterStageX11 *stage_x11;
   ClutterStage *stage;
   ClutterStageWindow *impl;
+  ClutterDeviceManager *manager;
   gboolean res, not_yet_handled = FALSE;
   Window xwindow, stage_xwindow;
-  ClutterDeviceManager *manager;
   ClutterInputDevice *device;
 
   backend_x11 = CLUTTER_BACKEND_X11 (backend);
@@ -462,8 +462,6 @@ event_translate (ClutterBackend *backend,
   if (stage == NULL)
     return FALSE;
 
-  manager = clutter_device_manager_get_default ();
-
   impl = _clutter_stage_get_window (stage);
   stage_x11 = CLUTTER_STAGE_X11 (impl);
   stage_xwindow = xwindow; /* clutter_x11_get_stage_window (stage); */
@@ -474,6 +472,8 @@ event_translate (ClutterBackend *backend,
 
   update_last_event_time (backend_x11, xevent);
 
+  manager = clutter_device_manager_get_default ();
+
   switch (xevent->type)
     {
     case ConfigureNotify:
@@ -485,6 +485,32 @@ event_translate (ClutterBackend *backend,
                         xevent->xconfigure.width,
                         xevent->xconfigure.height);
 
+          /* Queue a relayout - we want glViewport to be called
+           * with the correct values, and this is done in ClutterStage
+           * via _cogl_onscreen_clutter_backend_set_size ().
+           *
+           * We queue a relayout, because if this ConfigureNotify is
+           * in response to a size we set in the application, the
+           * set_size above is essentially a null-op.
+           *
+           * Make sure we do this only when the size has changed,
+           * otherwise we end up relayouting on window moves.
+           */
+          if ((stage_x11->state & CLUTTER_STAGE_STATE_FULLSCREEN) ||
+              (stage_x11->xwin_width != xevent->xconfigure.width) ||
+              (stage_x11->xwin_height != xevent->xconfigure.height))
+          clutter_actor_queue_relayout (CLUTTER_ACTOR (stage));
+
+          /* If we're fullscreened, we want these variables to
+           * represent the size of the window before it was set
+           * to fullscreen.
+           */
+          if (!(stage_x11->state & CLUTTER_STAGE_STATE_FULLSCREEN))
+            {
+              stage_x11->xwin_width = xevent->xconfigure.width;
+              stage_x11->xwin_height = xevent->xconfigure.height;
+            }
+
           clutter_actor_set_size (CLUTTER_ACTOR (stage),
                                   xevent->xconfigure.width,
                                   xevent->xconfigure.height);
@@ -542,6 +568,8 @@ event_translate (ClutterBackend *backend,
                   else
                     stage_x11->state &= ~CLUTTER_STAGE_STATE_FULLSCREEN;
 
+                  stage_x11->fullscreening = fullscreen_set;
+
                   event->type = CLUTTER_STAGE_STATE;
                   event->stage_state.changed_mask =
                     CLUTTER_STAGE_STATE_FULLSCREEN;
@@ -627,7 +655,9 @@ event_translate (ClutterBackend *backend,
 
     case KeyPress:
       event->key.type = event->type = CLUTTER_KEY_PRESS;
-      event->key.device = backend_x11->core_keyboard;
+      event->key.device =
+        clutter_device_manager_get_core_device (manager,
+                                                CLUTTER_KEYBOARD_DEVICE);
 
       translate_key_event (backend, event, xevent);
 
@@ -662,7 +692,9 @@ event_translate (ClutterBackend *backend,
         }
 
       event->key.type = event->type = CLUTTER_KEY_RELEASE;
-      event->key.device = backend_x11->core_keyboard;
+      event->key.device =
+        clutter_device_manager_get_core_device (manager,
+                                                CLUTTER_KEYBOARD_DEVICE);
 
       translate_key_event (backend, event, xevent);
       break;
@@ -676,7 +708,8 @@ event_translate (ClutterBackend *backend,
   /* Input device event handling.. */
   if (not_yet_handled)
     {
-      device = backend_x11->core_pointer;
+      device = clutter_device_manager_get_core_device (manager,
+                                                       CLUTTER_POINTER_DEVICE);
 
       /* Regular X event */
       switch (xevent->type)
@@ -964,6 +997,8 @@ static void
 events_queue (ClutterBackend *backend)
 {
   ClutterBackendX11 *backend_x11 = CLUTTER_BACKEND_X11 (backend);
+  ClutterBackendX11Class *backend_x11_class =
+    CLUTTER_BACKEND_X11_GET_CLASS (backend_x11);
   ClutterEvent      *event;
   Display           *xdisplay = backend_x11->xdpy;
   XEvent             xevent;
@@ -975,6 +1010,9 @@ events_queue (ClutterBackend *backend)
     {
       XNextEvent (xdisplay, &xevent);
 
+      if (backend_x11_class->handle_event (backend_x11, &xevent))
+        continue;
+
       event = clutter_event_new (CLUTTER_NOTHING);
 
       if (event_translate (backend, event, &xevent))
@@ -1010,13 +1048,14 @@ ClutterX11FilterReturn
 clutter_x11_handle_event (XEvent *xevent)
 {
   ClutterBackend      *backend;
+  ClutterBackendX11Class *backend_x11_class;
   ClutterEvent        *event;
   ClutterMainContext  *clutter_context;
   ClutterX11FilterReturn result;
   gint                 spin = 1;
 
   /* The return values here are someone approximate; we return
-   * CLUTTER_X11_FILTER_REMOVE if and only if a clutter event is
+   * CLUTTER_X11_FILTER_REMOVE if a clutter event is
    * generated for the event. This mostly, but not entirely,
    * corresponds to whether other event processing should be
    * excluded. As long as the stage window is not shared with another
@@ -1030,6 +1069,13 @@ clutter_x11_handle_event (XEvent *xevent)
 
   clutter_context = _clutter_context_get_default ();
   backend = clutter_context->backend;
+  backend_x11_class = CLUTTER_BACKEND_X11_GET_CLASS (backend);
+
+  /* If the backend just observed the event and didn't want it
+   * removed it could return FALSE, so assume that a TRUE return
+   * means that our caller should also do no further processing. */
+  if (backend_x11_class->handle_event (CLUTTER_BACKEND_X11(backend), xevent))
+    return CLUTTER_X11_FILTER_REMOVE;
 
   event = clutter_event_new (CLUTTER_NOTHING);
 
