@@ -20,22 +20,22 @@ typedef struct
 
 /* These variables are used instead of the standard GLSL variables on
    GLES 2 */
-#ifdef HAVE_COGL_GLES2
+#ifdef COGL_HAS_GLES
 
 #define GLES2_VARS \
   "precision mediump float;\n" \
-  "varying vec2 tex_coord;\n" \
+  "varying vec2 tex_coord[1];\n" \
   "varying vec4 frag_color;\n"
-#define TEX_COORD "tex_coord"
+#define TEX_COORD "tex_coord[0]"
 #define COLOR_VAR "frag_color"
 
-#else /* HAVE_COGL_GLES2 */
+#else /* COGL_HAS_GLES */
 
 #define GLES2_VARS ""
 #define TEX_COORD "gl_TexCoord[0]"
 #define COLOR_VAR "gl_Color"
 
-#endif /* HAVE_COGL_GLES2 */
+#endif /* COGL_HAS_GLES */
 
 /* a couple of boilerplate defines that are common amongst all the
  * sample shaders
@@ -277,6 +277,8 @@ set_shader_num (ClutterActor *actor, gint new_no)
 					            1.0f / tex_height);
   	    }
         }
+
+      g_object_unref (shader);
     }
 }
 
@@ -301,7 +303,7 @@ button_release_cb (ClutterActor    *actor,
   return FALSE;
 }
 
-#ifdef HAVE_COGL_GLES2
+#ifdef COGL_HAS_GLES
 static gboolean
 timeout_cb (gpointer data)
 {
@@ -314,7 +316,7 @@ timeout_cb (gpointer data)
 
   return TRUE;
 }
-#endif /* HAVE_COGL_GLES2 */
+#endif /* COGL_HAS_GLES */
 
 G_MODULE_EXPORT gint
 test_shader_main (gint argc, gchar *argv[])
@@ -402,6 +404,8 @@ test_shader_main (gint argc, gchar *argv[])
   clutter_actor_set_shader (actor, shader);
   clutter_actor_set_position (actor, 100, 100);
 
+  g_object_unref (shader);
+
   clutter_container_add_actor (CLUTTER_CONTAINER (stage), actor);
 
   clutter_actor_set_shader_param_int (actor, "tex", 0);
@@ -412,7 +416,7 @@ test_shader_main (gint argc, gchar *argv[])
   g_signal_connect (actor, "button-release-event",
                     G_CALLBACK (button_release_cb), NULL);
 
-#ifdef HAVE_COGL_GLES2
+#ifdef COGL_HAS_GLES
   /* On an embedded platform it is difficult to right click so we will
      cycle through the shaders automatically */
   g_timeout_add_seconds (3, timeout_cb, actor);
